/* -*- mode: c++ -*- */
#ifndef FR_ADC_DATA_I
#define FR_ADC_DATA_I

/** @cond IGNORE_BY_DOXYGEN */
%module frameCPP

%{
#include "general/types.hh"
#include "framecpp/FrAdcData.hh"
%}

%import "framecpp/FrameCPP.hh"
%import "framecpp/FrAdcData.hh"

%SharedPtr(FrAdcData)

class FrAdcData
{
public:
  typedef FR_ADC_DATA_CHANNELGROUP_TYPE channelGroup_type;
  typedef FR_ADC_DATA_CHANNELNUMBER_TYPE channelNumber_type;
  typedef FR_ADC_DATA_NBITS_TYPE nBits_type;
  typedef FR_ADC_DATA_BIAS_TYPE bias_type;
  typedef FR_ADC_DATA_SLOPE_TYPE slope_type;
  typedef FR_ADC_DATA_SAMPLERATE_TYPE sampleRate_type;
  typedef FR_ADC_DATA_TIMEOFFSET_TYPE timeOffset_type;
  typedef FR_ADC_DATA_FSHIFT_TYPE fShift_type;
  typedef FR_ADC_DATA_PHASE_TYPE phase_type;
  typedef FR_ADC_DATA_DATAVALID_TYPE dataValid_type;

#ifdef SWIGPYTHON
  typedef Container< FrVect > data_type;
#else
  using FrameCPP::FrProcData::data_type;
#endif

  FrAdcData( );
  FrAdcData( const std::string& Name,
	     channelGroup_type Group, channelNumber_type Channel,
	     nBits_type NBits,
	     sampleRate_type SampleRate );
	     
  const std::string& GetName() const; 
  const std::string& GetComment( ) const;
  channelNumber_type GetChannelNumber( ) const;
  channelGroup_type GetChannelGroup( ) const;
  nBits_type GetNBits( ) const;
  bias_type GetBias( ) const;
  slope_type GetSlope( ) const;
  // STRING GetUnits( ) const;
  sampleRate_type GetSampleRate( ) const;
  timeOffset_type GetTimeOffset( ) const;
  fShift_type GetFShift( ) const;
  phase_type GetPhase( ) const;
  dataValid_type GetDataValid( ) const;

  %extend {
    
    void
    AppendData( FrVect& Data )
    {
      self->RefData( ).append( Data );
    }

    //-------------------------------------------------------------------
    //-------------------------------------------------------------------
    size_t RefDataSize( );

    //-------------------------------------------------------------------
    //-------------------------------------------------------------------
    data_type& RefData( );
  }

  %insert("python") %{
    name = property( fget=lambda d:d.GetName( ) )
    comment = property( fget=lambda d:d.GetComment( ) )
    channelNumber = property( fget=lambda d:d.GetChannelNumber( ) )
    channelGroup = property( fget=lambda d:d.GetChannelGroup( ) )
    nBits = property( fget=lambda d:d.GetNBits( ) )
    bias = property( fget=lambda d:d.GetBias( ) )
    slope = property( fget=lambda d:d.GetSlope( ) )
    sampleRate = property( fget=lambda d:d.GetSampleRate( ) )
    timeOffset = property( fget=lambda d:d.GetTimeOffset( ) )
    fShift = property( fget=lambda d:d.GetFShift( ) )
    phase = property( fget=lambda d:d.GetPhase( ) )
    dataValid = property( fget=lambda d:d.GetDataValid( ) )
    data = property( fget=lambda d:d.RefData( ) )
  %}
};

%{
  FrAdcData::data_type&
  FrAdcData_RefData( FrAdcData* Adc )
  {
    return Adc->RefData( );
  }

  size_t
  FrAdcData_RefDataSize( FrAdcData* Adc )
  {
    return Adc->RefData( ).size( );
  }
%}

CONTAINER_WRAP(FrAdcData,FrAdcData)

/** @endcond */ /* IGNORE_BY_DOXYGEN */

#endif /* FR_ADC_DATA_I */
