#ifndef FrameCPP_VERSION_3_FrAdcData_HH
#define FrameCPP_VERSION_3_FrAdcData_HH

#if ! defined(SWIGIMPORTED)
#include <memory>
#include <vector>
#endif /* ! defined(SWIGIMPORTED) */

#include "framecpp/Common/Container.hh"

#include "framecpp/Version3/FrameSpec.hh"
#include "framecpp/Version3/Dimension.hh"
#include "framecpp/Version3/FrVect.hh"
#include "framecpp/Version3/STRING.hh"

namespace FrameCPP
{
  namespace Version_3
  {
    const REAL_4 FR_ADC_DATA_DEFAULT_BIAS = 0.0;
    const REAL_4 FR_ADC_DATA_DEFAULT_SLOPE = 1.0;
    const REAL_8 FR_ADC_DATA_DEFAULT_FSHIFT = 0.0;
    const INT_4S FR_ADC_DATA_DEFAULT_TIME_OFFSET_S = 0;
    const INT_4U FR_ADC_DATA_DEFAULT_TIME_OFFSET_N = 0;
    const INT_2U FR_ADC_DATA_DEFAULT_OVER_RANGE = 0;
    inline const char* FR_ADC_DATA_DEFAULT_UNITS( )
    {
      return "counts";
    }

    class FrAdcData
      : public Common::FrameSpec::Object
    {
    public:
      typedef Common::Container< FrVect >	data_type;
      typedef data_type::const_iterator const_iterator;
      typedef data_type::iterator iterator;

      typedef Common::Container< FrVect >	aux_type;
      typedef aux_type::const_iterator const_aux_iterator;
      typedef aux_type::iterator aux_iterator;

      //: Default constructor
      FrAdcData( );

      FrAdcData( const FrAdcData& Source );

      FrAdcData( const std::string& name, INT_4U crate,
		 INT_4U channel, INT_4U nBits,
		 REAL_8 sampleRate,
		 REAL_4 bias = FR_ADC_DATA_DEFAULT_BIAS,
		 REAL_4 slope = FR_ADC_DATA_DEFAULT_SLOPE,
		 const std::string& units = FR_ADC_DATA_DEFAULT_UNITS( ),
		 REAL_8 fShift = FR_ADC_DATA_DEFAULT_FSHIFT,
		 INT_4S timeOffsetS = FR_ADC_DATA_DEFAULT_TIME_OFFSET_S,
		 INT_4U timeOffsetN = FR_ADC_DATA_DEFAULT_TIME_OFFSET_N,
		 INT_2U overRange = FR_ADC_DATA_DEFAULT_OVER_RANGE );

      ~FrAdcData( );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      //: Comparison operator
      bool operator==( const FrAdcData& RHS ) const;
      bool operator!=( const FrAdcData& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

      //: Number of bytes needed to write this structure
      cmn_streamsize_type Bytes( const Common::StreamBase& Stream) const;

      virtual const char* const ObjectStructName( ) const;

      // Accessors
      const std::string& GetName() const; 
      const std::string& GetComment( ) const;
      INT_4U GetChannel( ) const;
      INT_4U GetCrate( ) const;
      INT_4U GetNBits( ) const;
      REAL_4 GetBias( ) const;
      REAL_4 GetSlope( ) const;
      STRING GetUnits( ) const;
      REAL_8 GetSampleRate( ) const;
      INT_4S GetTimeOffsetS( ) const;
      INT_4U GetTimeOffsetN( ) const;
      REAL_8 GetFShift( ) const;
      INT_2U GetOverRange( ) const;

      const aux_type& RefAux( ) const;
      aux_type& RefAux( );
      const data_type& RefData( ) const;
      data_type& RefData( );
   
      // Mutators
      void AppendComment( const std::string& comment );

      //: Merge with another FrAdcData
      FrAdcData& Merge( const FrAdcData& RHS );

      static promote_ret_type
      Promote( INT_2U Source,
	       promote_arg_type Obj,
	       Common::IStream* Stream );

      void SetOverRange( INT_2U flag );

      void SetBias( REAL_4 bias );

      void SetSlope( REAL_4 slope );

      void SetTimeOffset( INT_4S Seconds, INT_4U Nanoseconds );

      virtual FrAdcData* Create( Common::IStream& Stream ) const;

      //: Write the structure to the stream
      virtual void Write( Common::OStream& Stream ) const;

    protected:
      /// \brief Down grade an object
      virtual demote_ret_type
      demote( INT_2U Target,
	      demote_arg_type Obj,
	      Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual promote_ret_type
      promote( INT_2U Target, promote_arg_type Obj, Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      struct data_definition_type {
	STRING	name;
	STRING	comment;
	INT_4U	crate;
	INT_4U	channel;
	INT_4U	nBits;
	REAL_4	bias;
	REAL_4	slope;
	STRING	units;
	REAL_8	sampleRate;
	INT_4S	timeOffsetS;
	INT_4U	timeOffsetN;
	REAL_8	fShift;
	INT_2U	overRange;
	data_type	data;
	aux_type	aux;

	data_definition_type( );
	data_definition_type( const data_definition_type& Source );

	//: Comparison operator
	bool operator==( const data_definition_type& RHS ) const;
	bool operator!=( const data_definition_type& RHS ) const;

	void copy_core( const data_definition_type& Source );
      } m_data;

      FrAdcData( Common::IStream& Stream );
    };

    inline FrAdcData::promote_ret_type FrAdcData::
    Promote( INT_2U Source,
	     promote_arg_type Obj,
	     Common::IStream* Stream )
    {
      return Common::PromoteObject( DATA_FORMAT_VERSION, Source, Obj );
    }

    inline const char* const FrAdcData::
    StructName( )
    {
      static const char* const name = "FrAdcData";
      return name;
    }

    inline bool FrAdcData::data_definition_type::
    operator!=( const data_definition_type& RHS ) const
    {
      return ! ( *this == RHS );
    }

    inline void FrAdcData::
    AppendComment( const std::string& comment )
    {
      FrameCPP::Common::AppendComment( m_data.comment, comment );
    } 

    inline const std::string& FrAdcData::
    GetComment( ) const
    {
      return m_data.comment;
    }

    inline INT_4U FrAdcData::
    GetChannel( ) const
    {
      return m_data.channel;
    }

    inline INT_4U FrAdcData::
    GetCrate( ) const
    {
      return m_data.crate;
    }

    inline INT_4U FrAdcData::
    GetNBits( ) const
    {
      return m_data.nBits;
    }

    inline REAL_4 FrAdcData::
    GetBias( ) const
    {
      return m_data.bias;
    }

    inline REAL_4 FrAdcData::
    GetSlope( ) const
    {
      return m_data.slope;
    }

    inline STRING FrAdcData::
    GetUnits( ) const
    {
      return m_data.units;
    }

    inline REAL_8 FrAdcData::
    GetSampleRate( ) const
    {
      return m_data.sampleRate;
    }

    inline INT_4S FrAdcData::
    GetTimeOffsetS( ) const
    {
      return m_data.timeOffsetS;
    }

    inline INT_4U FrAdcData::
    GetTimeOffsetN( ) const
    {
      return m_data.timeOffsetN;
    }

    inline REAL_8 FrAdcData::
    GetFShift( ) const
    {
      return m_data.fShift;
    }

    inline INT_2U FrAdcData::
    GetOverRange( ) const
    {
      return m_data.overRange;
    }

    inline const FrAdcData::aux_type& FrAdcData::
    RefAux( ) const
    {
      return m_data.aux;
    }

    inline FrAdcData::aux_type& FrAdcData::
    RefAux( )
    {
      return m_data.aux;
    }

    inline const FrAdcData::data_type& FrAdcData::
    RefData( ) const
    {
      return m_data.data;
    }

    inline FrAdcData::data_type& FrAdcData::
    RefData( )
    {
      return m_data.data;
    }
   
    inline void FrAdcData::
    SetOverRange( INT_2U flag )
    {
      m_data.overRange = flag;
    }

    inline void FrAdcData::
    SetBias( REAL_4 bias )
    {
      m_data.bias = bias;
    }

    inline void FrAdcData::
    SetSlope( REAL_4 slope )
    {
      m_data.slope = slope;
    }

    inline void FrAdcData::
    SetTimeOffset( INT_4S Seconds, INT_4U Nanoseconds )
    {
      m_data.timeOffsetS = Seconds;
      m_data.timeOffsetN = Nanoseconds;
    }

    inline bool FrAdcData::
    operator==( const FrAdcData& RHS ) const
    {
      return ( this->m_data == RHS.m_data );
    }

    inline bool FrAdcData::
    operator!=( const FrAdcData& RHS ) const
    {
      return ! ( *this == RHS );
    }
  } // namespace - Version_3
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_3_FrAdcData_HH */
